#include "wx/wx.h"
#include <wx/glcanvas.h>
#include <GL/glu.h>

#define SPHERE 1
#define CUBE 2

class MyApp : public wxApp
{
public:
    virtual bool OnInit();
};

class TestGLCanvas;


class MyFrame : public wxFrame
{
public:
    MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
            const wxSize& size, long style = wxDEFAULT_FRAME_STYLE);

    void OnExit(wxCommandEvent& event);
    TestGLCanvas* m_canvas;

    DECLARE_EVENT_TABLE()
};


class TestGLCanvas: public wxGLCanvas
{
  friend class MyFrame;
public:
 TestGLCanvas(wxWindow *parent, const wxWindowID id = -1, 
    const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
    long style = 0, const wxString& name = "TestGLCanvas");
   
 ~TestGLCanvas(void);

 void OnPaint(wxPaintEvent& event);
 void OnSize(wxSizeEvent& event);
 void OnEraseBackground(wxEraseEvent& event);
 void OnEnterWindow( wxMouseEvent& event );
 void make_sphere();
 void make_cube();
 
 void Render( void );
 void InitGL(void);
   
private:
  bool   m_init;


DECLARE_EVENT_TABLE()
};


BEGIN_EVENT_TABLE(TestGLCanvas, wxGLCanvas)
  EVT_SIZE(TestGLCanvas::OnSize)
  EVT_PAINT(TestGLCanvas::OnPaint)
  EVT_ERASE_BACKGROUND(TestGLCanvas::OnEraseBackground)
  EVT_ENTER_WINDOW( TestGLCanvas::OnEnterWindow )
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
    EVT_MENU(wxID_EXIT, MyFrame::OnExit)
END_EVENT_TABLE()



IMPLEMENT_APP(MyApp)


static void Reshape(int width, int height)
{
  glViewport(0,0,width,height); 
  glMatrixMode( GL_PROJECTION ); 
  glLoadIdentity(); 
  glOrtho(-8,8,-8,8,-8,8); 


}


bool MyApp::OnInit()
{

  MyFrame *frame = new MyFrame(NULL, "Exemplo Basico", wxPoint(50, 50),
                               wxSize(350, 350));

  wxMenu *winMenu = new wxMenu;

  winMenu->Append(wxID_EXIT, "&Fechar");

  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(winMenu, "&Janela");

  frame->SetMenuBar(menuBar);

  frame->m_canvas = new TestGLCanvas(frame, -1, wxPoint(0, 0), wxSize(500, 500));

  frame->Show(TRUE);

  return TRUE;
}


MyFrame::MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
                 const wxSize& size, long style) 
         : wxFrame(frame, -1, title, pos, size, style)
{
    m_canvas = NULL;
}

TestGLCanvas::TestGLCanvas(wxWindow *parent, wxWindowID id,
    const wxPoint& pos, const wxSize& size, long style, const wxString& name):
  wxGLCanvas(parent, (wxGLCanvas*) NULL, id, pos, size, style, name )
{
    m_init = FALSE;
}


TestGLCanvas::~TestGLCanvas()
{
}

void MyFrame::OnExit(wxCommandEvent& event)
{
    Destroy();
}

void TestGLCanvas::OnEnterWindow( wxMouseEvent& event )
{
    SetFocus();
}

void TestGLCanvas::OnEraseBackground(wxEraseEvent& event)
{
}


void TestGLCanvas::OnPaint( wxPaintEvent& event )
{
    
        GLfloat especular[] = { 1.0, 1.0, 1.0, 1.0};
        GLfloat position [] = { 0.0, 3.0, 1.0, 0.0 };
        GLfloat position2[] = { 1.0, -2.0, 1.0, 0.0 };
        
        glShadeModel(GL_SMOOTH);
        glEnable(GL_DEPTH_TEST);
        glEnable(GL_COLOR_MATERIAL);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, especular);
        glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, 120.0);
        glLightfv (GL_LIGHT0, GL_SPECULAR, especular);
        glLightfv (GL_LIGHT0, GL_POSITION, position);
        glLightfv (GL_LIGHT1, GL_SPECULAR, especular);
        glLightfv (GL_LIGHT1, GL_POSITION, position2);
        glEnable(GL_LIGHTING);
        glEnable(GL_LIGHT0);
        glEnable(GL_LIGHT1);

        make_sphere();
        make_cube();


     Render();
     SwapBuffers();

}

void TestGLCanvas::make_sphere() 
{

        GLint slices, stacks;
        GLUquadricObj *quadObj;

        quadObj = gluNewQuadric();
        slices = 16;
        stacks = 10;

        glNewList(SPHERE, GL_COMPILE);

                glColor3f(1.0, 0.0, 1.0);
                gluSphere(quadObj, 4.0, slices, stacks);
        
        glEndList();
}        


void TestGLCanvas::make_cube() 
{

        glNewList(CUBE, GL_COMPILE);

                glColor3f(0.2,0.3, 0.4);
        
                glBegin(GL_QUAD_STRIP);
                        glNormal3d(0.0, 0.0, -1.0);
                        glVertex3d(2.5, 2.5, -2.5);
                        glVertex3d(2.5, -2.5, -2.5);
                        glVertex3d(-2.5, 2.5, -2.5);
                        glVertex3d(-2.5, -2.5, -2.5);
 
                        glNormal3d(-1.0, 0.0, 0.0);
                        glVertex3d(-2.5, 2.5, 2.5);
                        glVertex3d(-2.5, -2.5, 2.5);
                         
                        glNormal3d(0.0, 0.0, 1.0);
                        glVertex3d(2.5, 2.5, 2.5);
                        glVertex3d(2.5, -2.5, 2.5);

                        glNormal3d(1.0, 0.0, 0.0);
                        glVertex3d(2.5, 2.5, -2.5);
                        glVertex3d(2.5, -2.5, -2.5);
                glEnd();
        
        
                glBegin(GL_QUADS);

                        glNormal3d(0.0, 1.0, 0.0);
                        glVertex3d(-2.5, -2.5, -2.5);
                        glVertex3d(2.5, -2.5, -2.5);
                        glVertex3d(2.5, -2.5, 2.5);
                        glVertex3d(-2.5, -2.5, 2.5);
        
                        glNormal3d(0.0, 1.0, 0.0);
                        glVertex3d(-2.5, 2.5, -2.5);
                        glVertex3d(2.5, 2.5, -2.5);
                        glVertex3d(2.5, 2.5, 2.5);
                        glVertex3d(-2.5, 2.5, 2.5);
                glEnd();

        glEndList();
}

void TestGLCanvas::Render()
{
    wxPaintDC dc(this);

#ifndef __WXMOTIF__
    if (!GetContext()) return;
#endif

    SetCurrent();

    if (!m_init)
    {
        InitGL();
        m_init = TRUE;
    }

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glClear(GL_COLOR_BUFFER_BIT );
        glClearColor(0.0f,0.0f,0.0f,0.0f); 
        glColor3f(1.0, 0.0, 0.0);

        glMatrixMode( GL_MODELVIEW ); 
        glLoadIdentity();
        glTranslatef(3.5, 0.0, 0.0);

        glCallList(SPHERE);

        glTranslatef(-5.5, 0.0, 0.0);
        glRotatef(45, 1.0, 0.0, 0.0);
        glRotatef(45, 0.0, 1.0, 0.0);
        glRotatef(45, 0.0, 0.0, 1.0);
 
        glCallList(CUBE);

        glFlush();
        SwapBuffers();
}

void TestGLCanvas::InitGL()
{
   SetCurrent();
   glClearColor(0.0, 0.0, 0.0, 0.0);
   glEnable(GL_DEPTH_TEST);


}

void TestGLCanvas::OnSize(wxSizeEvent& event)
{
    int width, height;
    GetClientSize(& width, & height);
    Reshape(width, height);
        if (height==0)  
	{ 
                height=1;   
	}  


#ifndef __WXMOTIF__
    if (GetContext())
#endif
    {
        SetCurrent();
    }
}

